@@ -60,11 +60,14 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	// categoies
 	pNodeNumCats = (int*)CATNET_MALLOC(numnodes*sizeof(int));
-	if (!pNodeNumCats) 
+	if (!pNodeNumCats) {
+		UNPROTECT(1); //rSamples
 		return rvec;
+	}
 	pNodeCats = (int**)CATNET_MALLOC(numnodes*sizeof(int*));
 	if (!pNodeCats) { 
 		CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
 		return rvec;
 	}
 	memset(pNodeCats,    0, numnodes*sizeof(int*));
@@ -85,6 +88,7 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 		if (!pNodeCats[i]) {
 			CATNET_FREE(pNodeCats);
 			CATNET_FREE(pNodeNumCats);
+			UNPROTECT(1); //rSamples
 			return rvec;
 		}
 		for(j = 0; j < pNodeNumCats[i]; j++)
@@ -118,6 +122,7 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 	if (!pprobs || !klmat) {
 		CATNET_FREE(pNodeCats);
 		CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
 		return rvec;
 	}
 
@@ -127,7 +132,7 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 	pPerturbations = 0;
 	if(!isNull(rPerturbations)) {
 		PROTECT(rPerturbations = AS_INTEGER(rPerturbations));
-		pPerturbations = INTEGER(rPerturbations);
+		pPerturbations = INTEGER_POINTER(rPerturbations);
 		psubSamples = (int*)CATNET_MALLOC(numnodes*numsamples*sizeof(int));
 	}
 
@@ -193,9 +198,9 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 		}
 	}
 
-	UNPROTECT(1); //rSamples
 	if(!isNull(rPerturbations))
 		UNPROTECT(1); //rPerturbations
+	UNPROTECT(1); //rSamples
 
 	if(psubSamples)
 		CATNET_FREE(psubSamples);
@@ -227,7 +232,6 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 	//Rprintf(str);
 
 	return rvec;
-
 }
 
 
@@ -249,8 +253,8 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 	PROTECT(rSamples = AS_INTEGER(rSamples));
 	pSamples = INTEGER(rSamples);
 
-	dim = GET_DIM(rSamples);
-	numnodes = INTEGER(dim)[0];
+	dim        = GET_DIM(rSamples);
+	numnodes   = INTEGER(dim)[0];
 	numsamples = INTEGER(dim)[1];
 
 	// pSamples are assumed positive indices
@@ -274,6 +278,8 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 			CATNET_FREE(pNodeCats);
 		if (pNodeNumCats)
 			CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
+		return rvec;
 	}
 
 	memset(pNodeCats,    0, numnodes*sizeof(int*));
@@ -296,6 +302,7 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 			CATNET_FREE(pnoderanks);
 			CATNET_FREE(pNodeCats);
 			CATNET_FREE(pNodeNumCats);
+			UNPROTECT(1); //rSamples
 			return rvec;
 		}
 		for(j = 0; j < pNodeNumCats[i]; j++)
@@ -331,6 +338,7 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 		CATNET_FREE(pnoderanks);
 		CATNET_FREE(pNodeCats);
 		CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
 		return rvec;
 	}
 
@@ -340,7 +348,7 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 	pPerturbations = 0;
 	if(!isNull(rPerturbations)) {
 		PROTECT(rPerturbations = AS_INTEGER(rPerturbations));
-		pPerturbations = INTEGER(rPerturbations);
+		pPerturbations = INTEGER_POINTER(rPerturbations);
 		psubSamples = (int*)CATNET_MALLOC(numnodes*numsamples*sizeof(int));
 	}
 
@@ -406,9 +414,9 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 		}
 	}
 
-	UNPROTECT(1); //rSamples
 	if(!isNull(rPerturbations))
 		UNPROTECT(1); //rPerturbations
+	UNPROTECT(1); //rSamples
 
 	if(psubSamples)
 		CATNET_FREE(psubSamples);
@@ -456,7 +464,6 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 	//Rprintf(str);
 
 	return rvec;
-
 }
 
 SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
@@ -477,17 +484,15 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 	PROTECT(rSamples = AS_INTEGER(rSamples));
 	pSamples = INTEGER(rSamples);
 
-	dim = GET_DIM(rSamples);
-	numnodes = INTEGER(dim)[0];
+	dim        = GET_DIM(rSamples);
+	numnodes   = INTEGER(dim)[0];
 	numsamples = INTEGER(dim)[1];	
 
 	if(isNull(rPerturbations)) {
-		UNPROTECT(1); //rSamples
 		PROTECT(rvec = NEW_NUMERIC(numnodes*numnodes));
 		pvec = NUMERIC_POINTER(rvec);
 		memset(pvec, 0, numnodes*numnodes*sizeof(double));
 		UNPROTECT(1);
-		return rvec;
 	}
 
 	// pSamples are assumed positive indices
@@ -497,11 +502,14 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	// categoies
 	pNodeNumCats = (int*)CATNET_MALLOC(numnodes*sizeof(int));
-	if (!pNodeNumCats) 
+	if (!pNodeNumCats) {
+		UNPROTECT(1); //rSamples
 		return rvec;
+	}
 	pNodeCats = (int**)CATNET_MALLOC(numnodes*sizeof(int*));
 	if (!pNodeCats) { 
 		CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
 		return rvec;
 	}
 	memset(pNodeCats,    0, numnodes*sizeof(int*));
@@ -522,6 +530,7 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 		if (!pNodeCats[i]) {
 			CATNET_FREE(pNodeCats);
 			CATNET_FREE(pNodeNumCats);
+			UNPROTECT(1); //rSamples
 			return rvec;
 		}
 		for(j = 0; j < pNodeNumCats[i]; j++)
@@ -561,6 +570,7 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 			CATNET_FREE(pprobs2);
 		if (klmat) 
 			CATNET_FREE(klmat);
+		UNPROTECT(1); //rSamples
 		return rvec;
 	}
 
@@ -568,10 +578,9 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 	pPerturbations = 0;
 	if(!isNull(rPerturbations)) {
 		PROTECT(rPerturbations = AS_INTEGER(rPerturbations));
-		pPerturbations = INTEGER(rPerturbations);
+		pPerturbations = INTEGER_POINTER(rPerturbations);
 		pSamplesPert = (int*)CATNET_MALLOC(numnodes*numsamples*sizeof(int));
 	}
-
 	
 	memset(klmat, 0, numnodes*numnodes*sizeof(double));
 
@@ -637,9 +646,9 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 		}
 	}
 
-	UNPROTECT(1); //rSamples
 	if(!isNull(rPerturbations))
 		UNPROTECT(1); //rPerturbations
+	UNPROTECT(1); //rSamples
 
 	if(pSamplesPert)
 		CATNET_FREE(pSamplesPert);
@@ -692,17 +701,15 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 	PROTECT(rSamples = AS_INTEGER(rSamples));
 	pSamples = INTEGER(rSamples);
 
-	dim = GET_DIM(rSamples);
-	numnodes = INTEGER(dim)[0];
+	dim        = GET_DIM(rSamples);
+	numnodes   = INTEGER(dim)[0];
 	numsamples = INTEGER(dim)[1];	
 
 	if(isNull(rPerturbations)) {
-		UNPROTECT(1); //rSamples
 		PROTECT(rvec = NEW_NUMERIC(numnodes*numnodes));
 		pvec = NUMERIC_POINTER(rvec);
 		memset(pvec, 0, numnodes*numnodes*sizeof(double));
 		UNPROTECT(1);
-		return rvec;
 	}
 
 	// pSamples are assumed positive indices
@@ -712,11 +719,14 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	// categoies
 	pNodeNumCats = (int*)CATNET_MALLOC(numnodes*sizeof(int));
-	if (!pNodeNumCats) 
+	if (!pNodeNumCats) {
+		UNPROTECT(1); //rSamples
 		return rvec;
+	}
 	pNodeCats = (int**)CATNET_MALLOC(numnodes*sizeof(int*));
 	if (!pNodeCats) { 
 		CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
 		return rvec;
 	}
 	memset(pNodeCats,    0, numnodes*sizeof(int*));
@@ -737,6 +747,7 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 		if (!pNodeCats[i]) {
 			CATNET_FREE(pNodeCats);
 			CATNET_FREE(pNodeNumCats);
+			UNPROTECT(1); //rSamples
 			return rvec;
 		}
 		for(j = 0; j < pNodeNumCats[i]; j++)
@@ -776,6 +787,7 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 			CATNET_FREE(pprobs2);
 		if (klmat) 
 			CATNET_FREE(klmat);
+		UNPROTECT(1); //rSamples
 		return rvec;
 	}
 
@@ -783,7 +795,7 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 	pPerturbations = 0;
 	if(!isNull(rPerturbations)) {
 		PROTECT(rPerturbations = AS_INTEGER(rPerturbations));
-		pPerturbations = INTEGER(rPerturbations);
+		pPerturbations = INTEGER_POINTER(rPerturbations);
 		pSamplesPert   = (int*)CATNET_MALLOC(numnodes*numsamples*sizeof(int));
 	}
 
@@ -846,9 +858,9 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 		}
 	}
 
-	UNPROTECT(1); //rSamples
 	if(!isNull(rPerturbations))
 		UNPROTECT(1); //rPerturbations
+	UNPROTECT(1); //rSamples
 
 	if(pSamplesPert)
 		CATNET_FREE(pSamplesPert);
@@ -992,7 +1004,7 @@ double *catnetPairwiseCondLikelihood(SEXP rSamples, SEXP rPerturbations) {
 	pPerturbations = 0;
 	if(!isNull(rPerturbations)) {
 		PROTECT(rPerturbations = AS_INTEGER(rPerturbations));
-		pPerturbations = INTEGER(rPerturbations);
+		pPerturbations = INTEGER_POINTER(rPerturbations);
 		pSamplesPert = (int*)CATNET_MALLOC(numnodes*numsamples*sizeof(int));
 	}
 	
