@@ -530,7 +530,7 @@ SEXP RCatnetSearchSA::search(SEXP rNodeNames, SEXP rSamples,
 
 			if (!isNull(rPerturbations)) {
 				pPerturbations = m_pSearchParams[n]->m_pPerturbations;
-				pRperturbations = INTEGER(rPerturbations);
+				pRperturbations = INTEGER_POINTER(rPerturbations);
 				for (j = 0; j < m_numSamples; j++) {
 					for (i = 0; i < m_numNodes; i++)
 						pPerturbations[j * m_numNodes + i] = pRperturbations[j
